#  Lawson Labs, Inc.
#  Open Source Python 2.7 code for Model 201 and EMF16
#  Feb 6, 2017
#  www.lawsonlabs.com
#  lawsnlab@lawsonlabs.com
#  DEMO201rev1.py 
#
#  See Instructions.pdf for operational description
#
#  Other resources:
#  DRIVR201.BAS original, well remarked source code
#               with similar variables and parallel function
#
#  requires drivr201.cfg file in same folder for set up info
#                
#
#  NT_m201 is a C application with parallel function using a dll
#  The Model 201 Manual documents hardware, including the command structure
#
#  We welcome questions and comments and bug reports


import serial
import time
import msvcrt
import curses

#import logging
#LOG_FILENAME = 'debug.txt'
#$logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
#logging.debug('start of debug\n')

stdscr = curses.initscr()
curses.cbreak()
stdscr.keypad(1)
curses.echo()
curses.curs_set(0)

global CHECKSUM, rawcount, ser, version, SCALE16, SCALE24
global inbyte, rawcount, SCALE, BU, WORDCNT, STBY, lastin, lastout
global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, CHANR
global cnt, BAUDR, nums, NUMCCS, FTIME, Pof, adcount, WAITIME, nc, stypestr

STBY=0
CHECKSUM = 0
cnt = 0
rawcount=0
lastin="   "
lastout="   "
chks= "        "
st=""
nc = [0,0,0,0,0,0]
stypestr = [" Single Channel  ", "      Normal     ", "Self-Calibration"]

def getchar(timeout):   # waits for 1 character or timeout
    global CHECKSUM
    xx= time.time() + timeout
    while (time.time() < xx):
        cnt= ser.inWaiting()
        if (cnt > 0):
            inbyte = ser.read()
            CHECKSUM = (CHECKSUM + ord(inbyte)) % 256
            return inbyte         
    return -1

def echotest():
    echoByte = 1
    keyhit= ""
    while (keyhit == ""):
        ser.write(chr(echoByte))
        bchar = getchar(.4)
        if bchar == -1:
            return "timeout"
        if (ord(bchar) == echoByte):
            stdscr.move(11, 31)
            stdscr.addstr( "Echo byte %7d" % ord(bchar))
            stdscr.refresh()
            echoByte= ((echoByte) % 255) + 1       
        else:
            return "echo failure"
           
        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
           
    return "echo test success"

def fileread(fline, f):      #read next value from a comma-delimited file
    global cnt               # f is flag  1= return an integer value
    cf = fline.find(",", cnt)
    st= fline[cnt:cf]
    if f:
        st= int(st)
    cnt = cf + 1
    return st

def sendpacket(B1, B2):
    ser.write(chr(B1))
    ser.write(chr(B2))
    ser.write(chr((B1 + B2) % 256))

def sendgetdatacmd (B1, B2):
    global inbyte
    sendpacket(B1, B2)
    inbyte= getchar(1)
    if ord(inbyte) <> B1:
        print  ord(inbyte), B1, "command token echo error "
        
def getanumber(prompt):
    stdscr.move(22, 50)
    stdscr.addstr(prompt)
    ans = stdscr.getstr()
    stdscr.move(22, 50)
    stdscr.addstr( "                          ")
    acum= val(ans)
    return acum
 
def getanumberb(prompt):
    stdscr.move(22, 1)
    stdscr.addstr(prompt)
    ans = stdscr.getstr()
    stdscr.move(22, 0)
    stdscr.addstr( "                                                        ")
    stdscr.refresh()
    acum= val(ans)
    return acum

def val(data):   # same as the basic val() function returning positive values only   
    dpt=0        # initialize decimal point to end of string
    acum=0       # accumulated answer
    ct= 0        # number of digits
    nums= range(len(data)-1, -1, -1) # start at end of string
    for i in nums:                   # keep numerals only
        if (ord(data[i]) >47) and (ord(data[i]) < 58):
            acum = acum + int(data[i]) * 10**ct   # add weighted digit to total
            ct = ct + 1              # next digit 
        if data[i]==".":
            dpt= ct                  # got a decimal point                 
    acum= acum * 10 **-dpt           # adjust the order of magnitude
    return   acum 

def adjrate(RATE):                   # return the actual, achievable rate
    global MODEREGMID, MODEREGLO, WAITIME
    ADCLK = 19531.25                    # 10000000/512 for 10 MHz crystal
    MODECODE = int(ADCLK / RATE)        # calculate the mode register data
    MODEREGLO = MODECODE & 0xFF
    MODEREGMID = (MODECODE & 0x700) / 256
    RATE = ADCLK / (((MODEREGMID & 0x7) * 256) + MODEREGLO)
    MODEREGMID= MODEREGMID + WL         # restore word length
    WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
    return RATE

def getconver():    #  get an A/D conversion, calculate count and scale in volts
    inbyte = getchar(WAITIME)
    if inbyte <> -1:
        rawcount = ord(inbyte)
    else:
        rawcount = 0
        print "getconver timeout"
    inbyte = getchar(1)
    if inbyte <> -1:
        rawcount = rawcount + ord(inbyte) * 256
    else:
        print "getconver timeout1"
    if WL:
        inbyte = getchar(1)  
        rawcount = rawcount + ord(inbyte) * 65536
    VOLTS = (rawcount * SCALE)
    if BU == 0:
        VOLTS = VOLTS * 2 - 5
    return (VOLTS, rawcount)

def   syscal():            # perform system calibration
      global GAIN
      ORGAIN = GAIN
      if GAIN <> 1:
        GAIN = 1
        warmstart()
      #  IF gain not equal to 1, save original gain AND calibrate with GAIN 1
      CC = 7 * 16 
      DOCAL(CC, "O")
      CC = 6 * 16 
      DOCAL(CC, "F")
      if GAIN <> ORGAIN:
        GAIN = ORGAIN
        warmstart()
        CC = 7 * 16 
        DOCAL(CC, "O")
 

def   DOCAL(CC, A):   #  perform individual calibration on channel selected
      B2 = CC
      if A == "O":
          B1 = 0x82
      if A == "F":
          B1 = 0x83
      sendgetdatacmd (B1, B2)
      nums = range (0, WORDCNT)
      for i in nums:
          inbyte= getchar((WAITIME + FTIME)*2)   
         
def warmstart():               #  re-start after mode change
      global MODEREGHI        
      B1 = 0x84
      B2=  0x0 
      sendgetdatacmd (B1, B2)
      time.sleep(.1)
      MODEREGHI = ((GAIN-1) * 4) + STBY
      sendpacket (MODEREGHI, MODEREGMID)
      sendpacket (MODEREGLO, MODEREGLO)
      B1 = getchar(1)
      B2 = getchar(1)
      B3 = getchar(1)
      MODEREGHI = ((GAIN-1) * 4) + STBY
      if (ord(B1) <> MODEREGHI) or (ord(B2) <> MODEREGMID) or (ord(B3) <> MODEREGLO):
          print "no mode register echo "
          quit()
          
def checksum():       #do get checkum command and test result
      global CHECKSUM
      sendgetdatacmd (0x87, 1)
      inbyte= getchar(1)
      if CHECKSUM <> ((ord(inbyte) * 2) % 256):
          print " M201 checksum (HEX) ", ord(inbyte), " LOCAL checksum (HEX) ", CHECKSUM
          chks = "Chksum NG"
      else:
          chks= "Chksum OK"
      CHECKSUM = 0
      return chks
    
def CalcSD(avgcts):  # pass in count, return standard deviation in counts
    global SDCOUNT, SD, SDTOT, sdptr, sdcnt, SDGROUP
    SD[sdptr] = avgcts
    SDTOT = SDTOT + avgcts
    if sdptr <> SDGROUP:
        sdptr = sdptr + 1
        return sdcnt
    sdcnt = 0.0
    SDTOT = SDTOT / SDGROUP  # average of SDGROUP samples, usually 40
    nums = range(1, SDGROUP + 1)
    for N in nums:  
        sdcnt = sdcnt + ((SDTOT - SD[N]) ** 2)

    sdcnt = (sdcnt / (SDGROUP-1))** (1/2.0)
    sdptr=1
    SDTOT = 0
    if sdcnt> MAXBITS:
        sdcnt= MAXBITS 
    return sdcnt

    
def writeconfig():
    global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
    global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
    global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, SCALE16, SCALE24
    global cnt,BAUDR, nums, NUMCCS, WORDCNT, SCALE, FTIME, Pof, adcount, CHANR
    global CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5
    cfg=open ("drivr201.cfg", "w")
    cfg.write(str(MODEREGHI) + "," + str(MODEREGMID) + "," + str(MODEREGLO) + ",")
    cfg.write(str(CHANNEL) + "," + str(FILTER) + "," + str(LOCAVG)+ "\n")
    cfg.write(str(EXTCODE) + "," + str(BAUD) + "," + chr(34) + str(COM)+ ":" + chr(34)+ ",")
    cfg.write(str(INPBUF) + "," + str(NUMAVG) + "," + str(BU)+ ",")
    cfg.write(str(GAIN) + "," + str(RATE) + "," + str(WL)+ "\n")
    cfg.write(str(CHANR[0]) + "," + str(CHANR[1]) + "," + str(CHANR[2]) + ",")
    cfg.write(str(CHANR[3]) + "," + str(CHANR[4]) + "," + str(CHANR[5])+ "\n")
    cfg.write(str(SINGLECHAN) + "\n")
    cfg.write(str(SCAN) + "," + str(SCAN1) + "," + str(SCAN2) + ",")
    cfg.write(str(SCANINTERVAL) + "," + str(STYPE) + "," + str(RESERVE)+ "\n")
    cfg.close()

def readconfig():
    global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
    global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
    global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, SCALE16, SCALE24
    global cnt,BAUDR, nums, NUMCCS, WORDCNT, SCALE, FTIME, Pof, adcount, CHANR
    global CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5, WAITIME, MAXBITS

    cfg=open ("drivr201.cfg", "r")
    fline = cfg.readline()

    cnt=0
    MODEREGHI = fileread(fline, 1)
    MODEREGMID = fileread(fline, 1)
    MODEREGLO = fileread(fline, 1)
    CHANNEL = fileread(fline, 1)
    FILTER = fileread(fline, 1)
    LOCAVG = fileread(fline, 1)
    print MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG

    fline = cfg.readline()
    cnt=0
    EXTCODE = fileread(fline, 1)
    BAUD = fileread(fline, 1)
    COM = fileread(fline, 0)
    COM= COM[1:len(COM)-2]
    INPBUF = fileread(fline, 1)
    NUMAVG = fileread(fline, 1)
    BU= fileread(fline, 1)
    GAIN = fileread(fline, 1)
    RATE = fileread(fline, 0)
    WL = fileread(fline,1)
    RATE=float(RATE)
    GAIN= int(GAIN)
    print EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL

    fline = cfg.readline()
    cnt=0
    CHAN0 = fileread(fline, 1)
    CHAN1 = fileread(fline, 1)
    CHAN2 = fileread(fline, 1)
    CHAN3 = fileread(fline, 1)
    CHAN4 = fileread(fline, 1)
    CHAN5 = fileread(fline, 1)
    print CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5
    CHANR = [CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5]

    fline = cfg.readline()
    cnt=0
    SINGLECHAN = fileread(fline, 1)
    print SINGLECHAN

    fline = cfg.readline()
    cnt=0
    SCAN = fileread(fline, 1)
    SCAN1 = fileread(fline, 1)
    SCAN2 = fileread(fline, 1)
    SCANINTERVAL = fileread(fline, 0)
    STYPE = fileread(fline, 1)
    RESERVE = fileread(fline, 0)
    SCANINTERVAL = float(SCANINTERVAL)
    print SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE

    cfg.close()
    
    BAUDR =[9600, 4800, 2400, 1200, 600, 300]
    nums = range(0,6)
    NUMCCS = 0
    for N in nums:
        CC = CHANR[N] % 16  - int(CHANR[N] / 16) + 1
        NUMCCS = NUMCCS + CC
        nc[N] =CC                  # subchannels on this A/D channel  
    WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
    
    SCALE24 = float(5.0 / 2**24)   # one count's worth for 5 volt range
    SCALE16 = float(5.0 / 2**16)   # one count's worth for 16 bit mode
    if WL == 0:
        WORDCNT = 2
        SCALE= SCALE16
        MAXBITS=16
    else:
        WORDCNT = 3
        SCALE= SCALE24
        MAXBITS=24
    if LOCAVG == 0:
        LOCAVG = 1
    if FILTER == 0:
        FTIME = float(4.77)
    if FILTER == 1:
        FTIME = float(0.61)
    if FILTER == 2:
        FTIME = float(0.036)
 
    Pof=1                        # power of 2 of remote averaging
    while (NUMAVG / 2 ** Pof) > 1:
            Pof= Pof + 1

    if STYPE==0:
        NUMCCS=1
   
    MODEREGHI = MODEREGHI & 254   # strip off standby bit
  
def signon():
    global ser, CHECKSUM
    ser = serial.Serial(baudrate=300, bytesize=8, parity="N", stopbits=1, xonxoff=0, rtscts=0, dsrdtr=0, timeout=1, writeTimeout=2 )
    ser.port= COM
    ser.setRTS(0)
    ser.setDTR(1)
    ser.open()
    if not(ser):
        erstr="port NOT opened"
        return erstr
    token = b"\x00"
    nums = range(1, 21)

    for i in nums:
        time.sleep( .2 )
        ser.write(token)
        bchar = getchar(.5)
        if (bchar == b"\x03"):
            break
        if (i > 19):
            ser.close()
            erstr="can't wake"
            return erstr
        stdscr.move(20, 15 + i * 2)
        stdscr.addstr(" .")
        stdscr.refresh()
    bchar = ser.read()  # clears buffer
    CHECKSUM = 0
    time.sleep(.20)
    token = 0x88
    ser.write(chr(token))

    time.sleep(.10)
    ser.write(chr(BAUD))
    bchar = getchar(.3)
    if (bchar<>chr(BAUD)):     
        if (bchar == -1):
             erstr = "BAUD echo timeout"
             return erstr
        erstr = "BAUD echo wrong"
        return erstr
    ser.close()
    ser = serial.Serial (bytesize=8, parity='N', stopbits=1, xonxoff=0, timeout=1, writeTimeout=2 )
    ser.port=COM
    ser.baudrate=BAUDR[BAUD] 
    ser.setRTS(0)
    ser.setDTR(1)
    ser.open()
    if(ser):
        stdscr.addstr( "  port opened at %5s BAUD" % BAUDR[BAUD])
    else:
        erstr = "port NOT opened at selected BAUD"
        return erstr
    return "OK"

def finishsignon():
    global MODEREGHI, MODEREGMID, MODEREGLO
    token = b"\x00"            # end echo test
    ser.write(token)
    B1= MODEREGHI
    B2= MODEREGMID
    sendpacket(B1, B2)
    B1= MODEREGLO
    sendpacket(B1, B2)
    B1= Pof
    B2= FILTER
    sendpacket(B1, B2)
    B2= 0                       # scan mode
    sendpacket(B1, B2)
    B1 = getchar(1)
    B2 = getchar(1)
    B3 = getchar(1)
    if (ord(B1) <> MODEREGHI) or (ord(B2) <> MODEREGMID) or (ord(B3) <> MODEREGLO):
        erstr = "no mode register echo "
        return erstr
    B1 = SCAN                    # send scan interval and channels setup
    B2 = SCAN1
    sendpacket(B1, B2)
    B1 = SCAN2
    B2 = CHANR[0]
    sendpacket(B1, B2)
    B1 = CHANR[1]
    B2 = CHANR[2]
    sendpacket(B1, B2)
    B1 = CHANR[3]
    B2 = CHANR[4]
    sendpacket(B1, B2)
    B1 = CHANR[5]
    B2 = 0
    sendpacket(B1, B2)     

    return "OK"
    
def drawCmenu(): 
    stdscr.move(2, 22)
    stdscr.addstr("Communications Menu ")
    stdscr.move(4, 20)
    stdscr.addstr("Model 201/EMF16  2/2017")
    stdscr.move(7, 10)
    stdscr.addstr( "1) Com Port %6s" % COM)
    stdscr.move(9, 10)
    stdscr.addstr( "2) BAUD rate %5d" % BAUDR[BAUD])
    stdscr.move(11, 10)
    stdscr.addstr( "3) Echo Test")
    stdscr.move(13, 10)
    stdscr.addstr( "4) Main Menu")
    stdscr.move(15, 10)
    stdscr.addstr( "5) Save and Quit")
    stdscr.move(17, 10)
    stdscr.addstr( "6) No Save, just Quit")
    stdscr.move(19, 5)
    stdscr.refresh()
    
def drawpolled():
        # polled screen y,x  y 0-23  x 0-79 
    stb = ["OFF", "ON"]
    filt = ["  4Hz", " 40Hz", "400Hz"]
    wrd= ["16 bits", "24 bits"]
    sbu= ["Bipolar", "Unipolar"]
    stdscr.move(0, 31)
    stdscr.addstr("Main Menu ")
    stdscr.move(0, 61)
    stdscr.addstr( "Version %3d" % version)
    stdscr.move(2, 3)
    stdscr.addstr( "C) A/D input Channel %6d" % CHANNEL)
    stdscr.move(3, 3)
    stdscr.addstr( "E) External sub-channel %3d" % EXTCODE)
    stdscr.move(4, 3)
    stdscr.addstr( "A) remote Averaging %7d" % NUMAVG)
    stdscr.move(5, 3)
    stdscr.addstr( "L) Local averaging %8d" % LOCAVG)
    stdscr.move(6, 3)
    stdscr.addstr( "T) filTer %19s" % filt[FILTER])
    stdscr.move(7, 3)
    stdscr.addstr( "G) Gain %19d" % 2**(GAIN-1))
    stdscr.move(8, 3)
    stdscr.addstr( "R) internal data Rate %8.2f" % RATE)
    stdscr.move(9, 3)
    stdscr.addstr( "B) StandBy               " )
    if STBY:
        stdscr.addstr( "%3s" % stb[STBY], curses.A_REVERSE)   
    else:
        stdscr.addstr( "%3s" % stb[STBY], curses.A_NORMAL)
    stdscr.move(11, 3)
    stdscr.addstr( "N) to scaN", curses.A_NORMAL) 
    stdscr.move(2, 40)
    stdscr.addstr( "D) Digital input %12s" % lastin)
    stdscr.move(3, 40)
    stdscr.addstr( "U) digital oUtput %11s" % lastout)
    stdscr.move(4, 40)
    stdscr.addstr( "K) checKsum %18s" % chks)
    stdscr.move(5, 40)
    stdscr.addstr( "S) System calibration")
    stdscr.move(6, 40)
    stdscr.addstr( "O) Offset calibration") 
    stdscr.move(7, 40)
    stdscr.addstr( "W) Word length %16s" % wrd[int(WL/128)])
    stdscr.move(8, 40)
    stdscr.addstr( "P) Polarity %19s" % sbu[BU])
    stdscr.move(9, 40)
    stdscr.addstr( "Z) Sleep   " )
    stdscr.move(11,40)
    stdscr.addstr( "Q) to Quit") 
    stdscr.refresh()
    
def scan(doLog):
    global WAITIME
    if(doLog):
        startTime=time.strftime("%m-%d-%y %H:%M:%S")
        nm=time.strftime("%m%d%y_%H_%M_%S")
        nm= nm + ".txt"
        logFile=open (nm, "w")
        logFile.write("scan#\t")
        if(STYPE == 2):
            logFile.write(" OFFSET\t FULLSCALE\t")
        nums=range(1, NUMCCS + 1)
        for N in nums: 
            logFile.write(" %7d\t" % (N - 1))
        logFile.write("\n")
    idx=0
    if STYPE == 2:
        SCANBYTES = ((NUMCCS + 2) * WORDCNT) + 2
        B1 = 0x8C
            # calibrate scan token   &H8C
    if STYPE == 1:
        SCANBYTES = (NUMCCS * WORDCNT) + 2
        B1 = 0x89
            # normal scan command token
    if STYPE == 0:
        SCANBYTES = WORDCNT
            # singlechan has no start and end markers
        sendpacket (1, SINGLECHAN)
        time.sleep( .1)   
        B1 = 0x8B
            # single-channel scan token  &H8B
    sendgetdatacmd (B1, 0)
    stdscr.clear()
    stdscr.move(23,5)
    stdscr.addstr( "Press E to end scan") 
    stdscr.move(0,0)
    stdscr.addstr( "scan #               started at: " + startTime) 
    stdscr.refresh()
    moreinBuff=0
    ENDSCANSENT=0
    SCANCNT = 0
    WAITIME = SCANINTERVAL * 2
    while (ENDSCANSENT ==0) or moreinBuff:
        SCANCNT = SCANCNT + 1
        if(doLog):
            logFile.write("%6d\t" % SCANCNT)     
        
        if STYPE <> 0:
           inbyte=getchar(WAITIME)
           if ord(inbyte) <> 0xF0:
               if ord(inbyte) == 0x0a:
                   print "Scan interval too short"
               else:
                   print "Scan start framing error"
               quit()
        idx = (idx % 20) + 1
        stdscr.move(idx +1, 0)
        stdscr.addstr( "%2d " % SCANCNT)
        stdscr.refresh()     
        if STYPE == 2: 
            VOLTS, rawcount = getconver()
            OFFSET=VOLTS
            VOLTS, rawcount = getconver()
            FSVOLTS = VOLTS - OFFSET        
            stdscr.addstr( "  %2.6f " % OFFSET) if OFFSET >= 0 else stdscr.addstr( " %2.6f " % OFFSET)
            stdscr.addstr( "  %2.6f " % FSVOLTS) if FSVOLTS >= 0 else stdscr.addstr( " %2.6f " % FSVOLTS)
            if(doLog): 
                logFile.write(" %2.6f\t " % OFFSET) if OFFSET >= 0 else logFile.write( "%2.6f\t " % OFFSET)
                logFile.write(" %2.6f\t" % FSVOLTS) if FSVOLTS >= 0 else logFile.write( "%2.6f\t " % FSVOLTS)
            
        nums = range(0, NUMCCS)
        for N in nums: 
            VOLTS, rawcount = getconver()
            if STYPE==2:
                VOLTS=VOLTS-OFFSET
            stdscr.addstr( "  %2.6f " % VOLTS) if VOLTS >= 0 else stdscr.addstr( " %2.6f " % VOLTS)
            if(doLog): 
                logFile.write(" %2.6f\t" % VOLTS) if VOLTS >= 0 else logFile.write("%2.6f\t" % VOLTS)
            stdscr.refresh()             
        if(doLog): 
            logFile.write("\n")
        if STYPE <> 0:            #  should be end of scan token
            inbyte=getchar(1)
            if ord(inbyte) <> 0x0F:
              print "Scan framing failure (end of packet)", ord(inbyte)
              quit()

        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
            if keyhit == "E" or keyhit == "e":
                sendpacket(0x8A, 1)
                ENDSCANSENT=1
                time.sleep( .5) 
        morein = ser.inWaiting() 
        if morein > 1:
             moreinBuff = 1
        else:
             moreinBuff = 0
    inbyte= getchar(2)
   
    if ord(inbyte) == 0x8a:    
         erstr= "End scan OK"
    else:
        if ord(inbyte) == 0x0a:
            erstr= "interval too small "
        else:
            erstr= "No end scan token "
       
    print erstr
    if(doLog):
        endTime=time.strftime("%m-%d-%y %H:%M:%S")
        logFile.write("\nAveraging: %d" % NUMAVG)
        logFile.write("\nInternal Data Rate: %.2f" % RATE)
        logFile.write("\nNumber of Scans: %d" % SCANCNT)
        logFile.write("\nStarted: %s" % startTime)
        logFile.write("\nFinished: %s" % endTime)
        logFile.write("\n%s" % erstr)
        logFile.close()


def comMenu():
    global COM, BAUD, version, CMenu
    keyhit = ""
    Cmenu=1
    while (Cmenu == 1):
        drawCmenu()
        stdscr.move(20, 10)
        stdscr.refresh()
        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
        if keyhit== "6":
            curses.curs_set(1)
            quit()
        if keyhit== "1":
            TCOM= COM[len(COM)-1:len(COM)]
            COM= "COM" + str((ord(TCOM) % 8) + 1)
        if keyhit== "2":
            BAUD = (BAUD + 1) % 6
        if keyhit== "3":
            stdscr.addstr("waking   ")
            stdscr.refresh()
            erstr= signon()
            if erstr <> "OK":
                print erstr
                quit()
            stdscr.move(22, 10)
            stdscr.addstr("press a key to end echo test")
            result=  echotest()
            stdscr.clear()
            stdscr.move(11, 31)
            stdscr.addstr(result)
            stdscr.refresh()
        if keyhit== "4":
            stdscr.clear()
            stdscr.move(20, 10)
            stdscr.addstr("waking")
            stdscr.refresh()
            signon()
            result= finishsignon()
            print " sign on ", result   
            B1 = 0x86             # 86 is version command
            B2=  0
            sendgetdatacmd(B1, B2)
            inbyte= getchar(1)
            version= ord(inbyte)

            syscal()
            # restore original channel
            sendpacket (1, CHANNEL * 16 + EXTCODE)     # send control code 
            XX = (2.0 / RATE) + FTIME + .2       # WAIT for A/D to settle
            time.sleep( XX)

            stdscr.clear()
            stdscr.refresh()
            # polled screen y,x  y 0-23  x 0 to 79
            drawpolled()
            stdscr.move(21, 0)
            stdscr.refresh()
            #polled menu
            polled()
            stdscr.clear()
            stdscr.refresh()
        if keyhit== "5":
            writeconfig()
            curses.curs_set(1)
            quit()    
        keyhit=""
        
        
def polled():
    global CHECKSUM, rawcount, ser, version, SCALE16, SCALE24, MAXBITS
    global inbyte, rawcount, SCALE, BU, WORDCNT, STBY, lastin, lastout
    global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
    global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
    global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, sdptr, SD, SDTOT, sdcnt
    global cnt, BAUDR, nums, NUMCCS, FTIME, Pof, adcount, CMenu, WAITIME, SDGROUP
    adcount= []   # for local averaging
    avgptr=0      # circ buf ptr
    avgtot= 0
    avgv= float(0)
    sdptr=1
    SDTOT = 0
    SDGROUP= 40
    SD = []
    nums = range(0,SDGROUP+1)
    for i in nums:
        SD.append (0)
    sdcnt=MAXBITS       #standard deviation in counts
    keyhit = ""
    Pmenu=1
    idx=0
    sendgetdatacmd (0x81, 0)
    VOLTS, rawcount = getconver()
    nums = range(0,LOCAVG)
    for i in nums:
        adcount.append (rawcount)
        avgtot= avgtot + rawcount
    while (Pmenu == 1):
        drawpolled()
        stdscr.move(1, 22)
        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
            keyhit = keyhit.upper()
        if not STBY:
            sendgetdatacmd (0x81, 0)
            VOLTS, rawcount = getconver()
            if LOCAVG>1: 
                avgtot= avgtot + rawcount-adcount[avgptr]
                adcount[avgptr] = rawcount
                avgptr =(avgptr + 1) % LOCAVG
                avgcts= avgtot/LOCAVG
                avgv= avgcts * SCALE
                if BU == 0:
                   avgv = avgv * 2 - 5
            idx = (idx + 1) % 6
            stdscr.move((15 + idx), 2)
            stdscr.addstr( "Count %7d  Volts   %2.6f " % (rawcount, VOLTS)) if VOLTS >= 0 else stdscr.addstr( "Count %7d  Volts  %2.6f " % (rawcount, VOLTS))
            if LOCAVG>1:
                stdscr.addstr( " Avged   %2.6f" % (avgv)) if avgv >= 0 else stdscr.addstr( " Avged  %2.6f" % (avgv))
            else:
                avgv = VOLTS
                avgcts= rawcount
            sdcnt = CalcSD(avgcts)       # sdcnt = standard deviation in counts
            bits = sdcnt ** (1/2.0)      # bits = bits of RMS noise
            stdscr.addstr("                  ")
            stdscr.move (16, 58)
            stdscr.addstr("Effective")
            stdscr.move (17, 58)
            stdscr.addstr("Resolution ")
            stdscr.move (18, 58)
            stdscr.addstr("%2.1f in Bits " % (MAXBITS-bits))
            stdscr.refresh()
            if keyhit== "G":
                prompt= "Power of 2 for Gain? "
                ans = getanumber(prompt)
                GAIN= (int(ans) % 8)+ 1
                if GAIN<1:
                    GAIN=1
                stdscr.move(7, 28)
                stdscr.addstr("recal ")
                stdscr.refresh()
                warmstart()
                syscal()
                sendpacket (1, CHANNEL * 16 + EXTCODE)
                time.sleep(.5)
                stdscr.move(7, 28)
                stdscr.addstr("     ")
            if keyhit== "N":   # scan screen, then scanning setup, then scan()
                doScanMenu()
                sendpacket (1, CHANNEL * 16 + EXTCODE)
                WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
                stdscr.clear()
                stdscr.move(21, 3)
                stdscr.refresh()              
            if keyhit== "O":
                stdscr.move(6, 70)
                stdscr.addstr("cal ")
                stdscr.refresh()
                DOCAL(CHANNEL * 16 + EXTCODE, "O")
                time.sleep(1)
                stdscr.move(6, 70)
                stdscr.addstr("    ")  
            if keyhit== "P":
                BU= (BU + 1) % 2
                MODEREGMID= (MODEREGMID & 239) + BU * 16
                warmstart()
                syscal()
                sendpacket (1, CHANNEL * 16 + EXTCODE)               
            if keyhit== "R":
                prompt= "Rate, 10 to 1027? "
                ans = getanumber(prompt)
                RATE = float(ans)
                if RATE> 1027:
                    RATE = 1027
                if RATE< 10:
                    RATE = 10
                RATE = adjrate(RATE)
                WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
                warmstart()
                syscal()
                sendpacket (1, CHANNEL * 16 + EXTCODE)
            if keyhit== "S":
                stdscr.move(5, 70)
                stdscr.addstr("cal ")
                stdscr.refresh()
                syscal()
                sendpacket (1, CHANNEL * 16 + EXTCODE)
                time.sleep(1)
                stdscr.move(5, 70)
                stdscr.addstr("    ") 
            if keyhit== "W":
                WL= (WL + 128) % 256
                MODEREGMID= (MODEREGMID & 127) + WL
                warmstart()
                if WL == 0:
                    WORDCNT = 2
                    SCALE= SCALE16
                    MAXBITS=16
                else:
                    WORDCNT = 3
                    SCALE= SCALE24
                    MAXBITS=24
                syscal()
                sendpacket (1, CHANNEL * 16 + EXTCODE)
        if keyhit== chr(27):
             quit()
        if keyhit== "A":
            prompt= "Power of 2 for Avg? "
            ans = getanumber(prompt)
            Pof= int(ans) % 9
            NUMAVG= 2**Pof
            sendpacket (4, Pof)
            WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
        if keyhit== "B":
            STBY = (STBY + 1) % 2
        if keyhit== "E":
            prompt= "External code? "
            ans = getanumber(prompt)
            EXTCODE = int(ans) % 16
            sendpacket (1, CHANNEL * 16 + EXTCODE)
        if keyhit== "T":
            FILTER = (FILTER + 1) % 3
            sendpacket (3, FILTER)
            if FILTER == 0:
                FTIME = float(4.77)
            if FILTER == 1:
                FTIME = float(0.61)
            if FILTER == 2:
                FTIME = float(0.036)
        if keyhit== "U":
            prompt= "digital oUt ? "
            ans = getanumber(prompt)
            last= int(ans) % 256
            sendpacket (2, last)
            lastout= ""
            nums = range(7,-1,-1)
            for N in nums: 
                 if last & (2**N):      
                     lastout = lastout + "1"
                 else:
                     lastout = lastout + "0"
        if keyhit== "C":
            prompt= "Channel? "
            ans = getanumber(prompt) 
            CHANNEL = int(ans) % 8
            sendpacket (1, CHANNEL * 16 + EXTCODE)
        if keyhit== "D":
            sendgetdatacmd (0x80, 0x4C)
            digbyte= ord(getchar(1))
            lastin= "" 
            nums = range(7,-1,-1)
            for N in nums: 
                 if digbyte & (2**N):      
                     lastin = lastin + "1"
                 else:
                     lastin = lastin + "0"
        if keyhit== "K":
            stdscr.move(4, 63)
            stdscr.addstr(checksum()) 
        if keyhit== "L":
            prompt= "Local Average? "
            ans = getanumber(prompt)
            LOCAVG = int(ans) % 255
            if LOCAVG < 1:
                LOCAVG=1
            nums = range(1,LOCAVG+1)
            avgtot= 0
            for i in nums:
                adcount.append (rawcount)
                avgtot= avgtot + rawcount
        if keyhit== "Q":
            Pmenu=0
            Cmenu=1
        if keyhit== "Z":
            sendgetdatacmd (0x88, 0) 
            Pmenu=0
            Cmenu=1
        keyhit=""

    return

def CheckScanTime(getinput):
    global Pof, NUMAVG, SCAN, SCAN1, SCAN2, FTIME, BAUD
    global SCANINTERVAL, FILTER, STYPE, NUMCCS, RATE, WORDCNT

    baudTime = 0
    secRoll = 0
    rollScan = 0
    minTime = 0
    maxTime = 0

    while(True):
        minTime = (.007 * (2**BAUD)) + (NUMCCS * (((FTIME * .83) + ((NUMAVG + 2) / RATE))))
        baudTime = .0011 * (2**BAUD) * (NUMCCS * WORDCNT + 2)
        if STYPE == 2:                      # self-cal
          minTime = minTime + (.0042 * (2 ** BAUD)) + 2 * ((FTIME * .83) + 4 / RATE)
          baudTime = .0011 * (2 ** BAUD) * ((NUMCCS + 2) * WORDCNT)

        if (STYPE == 0):    # single-channel
            minTime = NUMAVG / RATE + (.0005 * (BAUD + 1))
            baudTime = .0011 * (2**BAUD) * WORDCNT

        maxTime = 4233 * (2**BAUD)       #maximum SCAN interval (secs) adjusted for baud rate
        secRoll = 3906.25 / (2**BAUD)    # of rollovers for 1 second adjusted for baud rate

        if(minTime < baudTime):
            minTime = baudTime            # baud rate limited SCAN

        if(minTime > maxTime):
            NUMAVG -= 1
            # send the average
            NUMAVG = 2**Pof
            sendpacket (4, Pof)
            continue
        break 
    minTime = (long)((minTime + .00005) * secRoll) / secRoll
    maxTime = int(maxTime * secRoll) / secRoll
    if getinput or SCANINTERVAL <  minTime or SCANINTERVAL >  maxTime:
        prompt= "enter an interval, %.4f to %.4f: " % (minTime, maxTime)
        ans = getanumberb(prompt)
        anstemp = float(ans)
        SCANINTERVAL = anstemp
        if(SCANINTERVAL < minTime):
            SCANINTERVAL = minTime
        if(SCANINTERVAL > maxTime):
            SCANINTERVAL = maxTime            
        rollScan = secRoll * (SCANINTERVAL + .00005) - 1      # rollovers per SCAN
        SCAN2 = int(rollScan / 65536)                         # calculate SCAN interval vars
        rollScan -= (SCAN2 * 65536)
        SCAN1 = int(rollScan / 256)                            #put in M201 format
        rollScan -= (SCAN1 * 256)
        SCAN = int(rollScan)
        SCANINTERVAL = (SCAN + SCAN1 * 256 + SCAN2 * 65536 + 1) / secRoll   
    return    

def drawScanMenu():
    stdscr.move(1, 26)
    stdscr.addstr("SCANNING  CONFIGURATION")
    stdscr.move(3, 13)
    if (STYPE <> 0):
        stdscr.addstr( "CHANNEL       EXTERNAL CONTROL CODE      DATA POINTS")
        stdscr.move(4, 28)
        stdscr.addstr( "FROM           TO         THIS CHANNEL")    
        stdscr.move(6, 16)
        stdscr.addstr( "0)           %2d            %2d            %2d" % (CHANR[0] / 0x10, CHANR[0] & 0xf, nc[0]))
        stdscr.move(7, 16)
        stdscr.addstr( "1)           %2d            %2d            %2d" % (CHANR[1] / 0x10, CHANR[1] & 0xf, nc[1]))
        stdscr.move(8, 16)
        stdscr.addstr( "2)           %2d            %2d            %2d" % (CHANR[2] / 0x10, CHANR[2] & 0xf, nc[2]))
        stdscr.move(9, 16)
        stdscr.addstr( "3)           %2d            %2d            %2d" % (CHANR[3] / 0x10, CHANR[3] & 0xf, nc[3]))
        stdscr.move(10, 16)
        stdscr.addstr( "4)           %2d            %2d            %2d" % (CHANR[4] / 0x10, CHANR[4] & 0xf, nc[4]))
        stdscr.move(11, 16)
        stdscr.addstr( "5)           %2d            %2d            %2d" % (CHANR[5] / 0x10, CHANR[5] & 0xf, nc[5]))
    else:
        stdscr.move(4, 26)
        stdscr.addstr( " CHANNEL      EXT CODE")
        stdscr.move(6, 26)
        stdscr.addstr("   %2d            %2d" % (SINGLECHAN / 0x10, SINGLECHAN & 0xf))
        stdscr.move(9, 16)
        stdscr.addstr( "S) Single channel" )
        stdscr.move(11, 16)
        stdscr.addstr( "E) Ext sub-channel" )   
    stdscr.move(13, 16)
    stdscr.addstr( "6) Scan Interval      %.4f  seconds     " % SCANINTERVAL)
    stdscr.move(15, 16)
    stdscr.addstr( "7) Scan type     %2s " % stypestr[STYPE])
    stdscr.move(17, 12)
    stdscr.addstr( "Enter) Perform the SCAN")
    stdscr.move(19, 12)
    tmp=NUMCCS + sc
    stdscr.addstr( "X) eXit SCAN menu     data points per scan: %3d" % tmp)
    stdscr.refresh()
  

def doScanMenu():
    global CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5, nc, NUMCCS, STYPE, SCANINTERVAL
    global SCAN, SCAN1, SCAN2, SINGLECHAN, CHANR, sc
    stdscr.clear()
    stdscr.refresh()
    fScanMenu = 1
    fsignon=0
    SCANINTERVAL = (SCAN + SCAN1 * 256 + SCAN2 * 65536 + 1) / (3906.25 / (2**BAUD))
    if STYPE==2:
        sc=2
    else:
        sc=0
    keyhit =""
    while (fScanMenu == 1):
        drawScanMenu()
        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
            keyhit = keyhit.upper()      
        if keyhit == "X":
            if fsignon:
                CheckScanTime(0)
                stdscr.clear()
                stdscr.move(20, 10)
                stdscr.addstr( "Waking")
                stdscr.refresh()    
                signon()
                finishsignon()
                syscal()
            stdscr.clear()
            stdscr.refresh()
            return
        if keyhit== chr(13):
            if fsignon:
                CheckScanTime(0)
                stdscr.clear()
                stdscr.move(20, 10)
                stdscr.addstr( "Waking")
                stdscr.refresh()    
                signon()
                finishsignon()
                syscal()
                if STYPE == 0:
                    sendpacket (1, SINGLECHAN)     # send control code 
                    XX = (2.0 / RATE) + FTIME + .2       # WAIT for A/D to settle
                    time.sleep( XX)
                fsignon=0   
            scan(1)       # pass zero to stop logging to disk
            stdscr.clear()
            stdscr.refresh()
        if len(keyhit)>0:
            i= ord(keyhit) - 48    
            if i >= 0 and i <=5:
                prompt= "enter starting control code for channel " + keyhit + " : "
                ans = getanumberb(prompt)
                stCODE = int(ans) % 16
                prompt= "enter ending control code for channel " + keyhit + " : "
                ans = getanumberb(prompt)
                endCODE = int(ans) % 16
                if endCODE < stCODE:
                    endCODE = 0
                    stCODE = 1
                CHANR[i] = stCODE * 16 + endCODE
                nc[i]= (endCODE-stCODE) + 1
                keyhit = ""
                fsignon=1
        if keyhit == "S":
            prompt= "Enter Single Channel code "
            ans = getanumberb(prompt) 
            CHAN = int(ans) % 256
            if CHAN < 0:
                CHAN = 0
            SINGLECHAN = (SINGLECHAN & 0x0f) + CHAN * 16   
        if keyhit == "E":
            prompt= "Enter External sub-channel "
            ans = getanumberb(prompt) 
            CHAN = int(ans) % 256
            if CHAN < 0:
                CHAN = 0
            SINGLECHAN = (SINGLECHAN & 0xF0) + CHAN     
        if keyhit == "6":
            CheckScanTime(1)
            fsignon=1
        if keyhit == "7":
            STYPE= (STYPE + 1) % 3
            if STYPE==2:    
                 sc = 2
            else:
                sc=0
            fsignon=1
            stdscr.clear()
            stdscr.refresh()
        NUMCCS = 0
        nums = range(0,6)
        for N in nums:
            NUMCCS = NUMCCS + nc[N]
        if STYPE== 0:
            NUMCCS=1
        keyhit = ""  

 
print ("Start time : %s" % time.ctime())
        
readconfig()
comMenu()

# dead code below
quit()



